#!/usr/bin/env python
# Written July 2008 by John Beaven <johnbeaven@users.sourceforge.net>
# a simple app for openmoko that auto connects to known APs..
# scans the visible APs and generates a wpa_supplicant.conf using configured
# passwords for accessible networks..
#
# GPL'd for her pleasure..
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


from pythonwifi.iwlibs import Wireless, getNICnames
from ConfigParser import *
import os

def main():
    
    config = ConfigParser()
    config.read('/etc/mofi/mofi.conf')
    
    # get a list of interfaces..
    ifnames = getNICnames()
    if ifnames == []:
        print "no WLAN devices found"
        return 1
        
    found_aps = {}
        
    # search for accesspoints the interfaces can access..
    for ifname in ifnames:
        
        wifi = Wireless(ifname)
        essid = wifi.getEssid()
        
        if not essid == '':
            print "scanning %s (connected to %s)" %(ifname, essid)
        else:
            print "scanning %s" %(ifname)
            
        wifi.scan()
        for results in wifi.scan():
            
            # see if we have a config setting for this essid..
            password = ''
            if config.has_option(results.essid, 'password'):
                password = str(config.get(results.essid, 'password'))
                print "  found %s (with known password)" %(results.essid)
            else:
                print "  found %s" %(results.essid)
                
            found_aps[results.essid] = {'password': password}
            
            #print "ESSID:", results.essid
            #print "PASSWORD:", password
            #print "Access point:", results.bssid
            #print "Mode:", results.mode
            #if len(results.rate) > 0:
            #    print "Highest Bitrate:", results.rate[len(results.rate)-1]
            #print "Quality: Quality ", results.quality.quality, "Signal ", results.quality.getSignallevel(), " Noise ", results.quality.getNoiselevel()
            
            # seems encryption [0x0,0x0,0x0,0x8] is WPA, [0x0,0x0,0x0,0x80] is open
            print "Encryption:", map(lambda x: hex(ord(x)), results.encode)
            
            
    print ''
    print 'generating wpa_supplicant configuration'
    
    prefered_order = []
    
    if config.has_option('DEFAULT', 'prefered_order'):
        prefered_order = str(config.get('DEFAULT', 'prefered_order')).split(',')
        
    # generate wpa_supplicant.conf from retrieved data..
    # if we have no pass, just assume open network..
    wpa_conf = """ctrl_interface=/var/run/wpa_supplicant
eapol_version=1
ap_scan=1
"""

	# prioritise aps in prefered_order from conf..
    for aps in prefered_order:
    	
    	if aps in found_aps:
    		
    		wpa_conf += """
network={
       ssid="%s"
       scan_ssid=1
       key_mgmt=WPA-PSK
       pairwise=CCMP TKIP
       group=TKIP CCMP
       psk="%s"
       priority=10
}""" %(aps, found_aps[aps]['password'])
    		
    		del found_aps[aps]

	# finally gen open connections for remaining aps..
    for aps in found_aps:
    	
    	wpa_conf += """
network={
       ssid="%s"
       key_mgmt=NONE
       priority=5
}""" %(aps)
    		

	# save config to file
	FILE = open('/etc/mofi/mofi_wpa_supplicant.conf',"w")
	FILE.write(wpa_conf)
	FILE.close()
	
	print 'connecting using wpa_supplicant configuration'
	
    # run shell script that uses conf to connect..
    os.system('/etc/mofi/connect.sh') 
    
if __name__ == "__main__":
    main()